#!/usr/bin/env python2.7
#
# KUAL Font Overrides menu rebuild
#
# $Id: rebuild_menu.py 15615 2018-12-18 19:21:53Z NiLuJe $
#
##

import sys, os, subprocess, syslog
from string import Template
# Requires a Python snapshot circa 0.15.N-r15585
from _fbink import ffi, lib as fbink

# NOTE: Hardcode HACKNAME for now
HACKNAME="linkfonts"

# We'll need this to kill stderr
DEVNULL = open(os.devnull, 'wb')
# NOTE: Use subprocess.DEVNULL w/ Python >= 3.3

# Setup FBInk to our liking...
FBINK_CFG = ffi.new("FBInkConfig *")
FBINK_CFG.is_quiet = True
# Centered & padded on both sides
FBINK_CFG.is_padded = True
FBINK_CFG.is_centered = True
# We want it near the bottom of the screen
FBINK_CFG.row = -4

# And initialize it
fbink.fbink_init(fbink.FBFD_AUTO, FBINK_CFG)


def kh_msg(msg, level='I', show='a', fbink_msg=None):
	# Check if we want to trigger an additionnal FBInk print
	if show == 'q':
		show_fbink=False
	elif show == 'v':
		show_fbink=True
	else:
		if os.path.isfile("/mnt/us/{}/verbose".format(HACKNAME)):
			show_fbink=True
		else:
			show_fbink=False

	# Unless we specified a different message, print the full message via FBInk
	if not fbink_msg:
		fbink_msg=msg

	# Setup syslog
	syslog.openlog('system: {} {}:kh_msg::'.format(level, HACKNAME))
	if level == "E":
		priority = syslog.LOG_ERR
	elif level == "W":
		priority = syslog.LOG_WARNING
	else:
		priority = syslog.LOG_INFO
	priority |= syslog.LOG_LOCAL4
	# Print to log
	syslog.syslog(priority, msg)

	# Do we want to trigger an FBInk print?
	if show_fbink:
		if HACKNAME == "linkfonts":
			fbink_tag="F"
		else:
			fbink_tag="U"

		# If loglevel is anything else than I, add it to our tag
		if level != "I":
			fbink_tag+=" {}".format(level)

		# Tag our message
		fbink_msg="{} {}".format(fbink_tag, fbink_msg)

		# And print it using FBInk (via cFFI)
		fbink.fbink_print(fbink.FBFD_AUTO, fbink_msg, FBINK_CFG)


## And now we can start!
# Check the extent to which we want to build the menu (empty, custom, full?)
if sys.argv[1] == "empty":
	menu_type="empty"
elif sys.argv[1] == "custom":
	menu_type="custom"
else:
	menu_type="full"

if menu_type != "empty":
	# We'll need a list of our custom fonts... It can take a while, so print a status message.
	kh_msg("Building fonts lists . . .", 'I', 'a')

	# Don't use our custom fc-list build, it expects a newer cache version. Use fc-list instead of fc-scan, it's much faster.
	# NOTE: This obeys the fontconfig config, so, we won't find the CJK fonts here if we hid them ;).
	fc_list_out=subprocess.check_output(["fc-list", "-f", "%{file}:%{family[0]}\n", ":", "file", "family" ], stderr=DEVNULL)

	# Make this a list...
	raw_fonts_list=fc_list_out.split('\n')

	# Remove the empty item we got from the trailing LF...
	raw_fonts_list.remove('')

	# Separate custom fonts from amazon fonts...
	amz_fonts = list()
	cus_fonts = list()
	for font in raw_fonts_list:
		if font.startswith("/usr/java/lib/fonts/"):
			# Since we used a separator we're pretty sure we won't find in a filename (':'), our family name is everything's that after the first colon we find ;)
			amz_fonts.append(font[font.find(":")+1:])
		else:
			cus_fonts.append(font[font.find(":")+1:])

	# And sort them, unique
	custom_fonts_list=sorted(set(cus_fonts))
	amazon_fonts_list=sorted(set(amz_fonts))

	# We'll need the index of the last item for our loop
	custom_fonts_last_id=len(custom_fonts_list) - 1
	amazon_fonts_last_id=len(amazon_fonts_list) - 1

# Here we go...
kh_msg("Building {} menu . . .".format(menu_type), 'I', 'a')

# Now the fun begins!
# Store the raw templates once
with open('/mnt/us/linkfonts/etc/kual-menu.tpl', 'r') as f:
	menu_template=Template(f.read())

if menu_type != "empty":
	with open('/mnt/us/linkfonts/etc/kual-submenu-item.tpl', 'r') as f:
		item_template=Template(f.read())

if menu_type == "full":
	# Build a full Amazon submenu
	amz_submenu=""
	for i, font in enumerate(amazon_fonts_list):
		# Don't add a comma to the last item
		if i != amazon_fonts_last_id:
			trailing_comma=','
		else:
			trailing_comma=''

		# Build an item for this specific font from our template. Don't touch TO_OVERRIDE, we'll handle it later.
		amz_submenu+=item_template.safe_substitute(TO_USE=font, COMMA=trailing_comma)

	# Now that we have it, make it a template, so we can handle TO_OVERRIDE later.
	amz_submenu_template=Template(amz_submenu)

if menu_type != "empty":
	# Build a full Custom submenu
	cus_submenu=""
	for i, font in enumerate(custom_fonts_list):
		# Don't add a comma to the last item
		if i != custom_fonts_last_id:
			trailing_comma=','
		else:
			trailing_comma=''

		# Build an item for this specific font from our template. Don't touch TO_OVERRIDE, we'll handle it later.
		cus_submenu+=item_template.safe_substitute(TO_USE=font, COMMA=trailing_comma)

	# Now that we have it, make it a template, so we can handle TO_OVERRIDE later.
	cus_submenu_template=Template(cus_submenu)

# And now we can build the final, complete menu! Handle each TO_OVERRIDE as we go.
if menu_type == "empty":
	full_menu=menu_template.safe_substitute(AMAZON_SERIF_SUB='', CUSTOM_SERIF_SUB='', AMAZON_SANS_SUB='', CUSTOM_SANS_SUB='', AMAZON_MONO_SUB='', CUSTOM_MONO_SUB='', AMAZON_CONDENSED_SUB='', CUSTOM_CONDENSED_SUB='', AMAZON_FALLBACK_SUB='', CUSTOM_FALLBACK_SUB='')
elif menu_type == "custom":
	full_menu=menu_template.safe_substitute(AMAZON_SERIF_SUB='', CUSTOM_SERIF_SUB=cus_submenu_template.safe_substitute(TO_OVERRIDE='serif'), AMAZON_SANS_SUB='', CUSTOM_SANS_SUB=cus_submenu_template.safe_substitute(TO_OVERRIDE='sans'), AMAZON_MONO_SUB='', CUSTOM_MONO_SUB=cus_submenu_template.safe_substitute(TO_OVERRIDE='mono'), AMAZON_CONDENSED_SUB='', CUSTOM_CONDENSED_SUB=cus_submenu_template.safe_substitute(TO_OVERRIDE='condensed'), AMAZON_FALLBACK_SUB='', CUSTOM_FALLBACK_SUB=cus_submenu_template.safe_substitute(TO_OVERRIDE='fallback'))
else:
	full_menu=menu_template.safe_substitute(AMAZON_SERIF_SUB=amz_submenu_template.safe_substitute(TO_OVERRIDE='serif'), CUSTOM_SERIF_SUB=cus_submenu_template.safe_substitute(TO_OVERRIDE='serif'), AMAZON_SANS_SUB=amz_submenu_template.safe_substitute(TO_OVERRIDE='sans'), CUSTOM_SANS_SUB=cus_submenu_template.safe_substitute(TO_OVERRIDE='sans'), AMAZON_MONO_SUB=amz_submenu_template.safe_substitute(TO_OVERRIDE='mono'), CUSTOM_MONO_SUB=cus_submenu_template.safe_substitute(TO_OVERRIDE='mono'), AMAZON_CONDENSED_SUB=amz_submenu_template.safe_substitute(TO_OVERRIDE='condensed'), CUSTOM_CONDENSED_SUB=cus_submenu_template.safe_substitute(TO_OVERRIDE='condensed'), AMAZON_FALLBACK_SUB=amz_submenu_template.safe_substitute(TO_OVERRIDE='fallback'), CUSTOM_FALLBACK_SUB=cus_submenu_template.safe_substitute(TO_OVERRIDE='fallback'))

# And then write it!
with open("/mnt/us/extensions/linkfonts-ovr/menu.json", "w") as f:
	f.write(full_menu)

kh_msg("Finished rebuilding menu", 'I', 'v')
