#!/bin/sh
#
# Fonts installer
#
# $Id: install.sh 12470 2015-09-27 00:13:54Z NiLuJe $
#
##

HACKNAME="linkfonts"

# Pull libOTAUtils for logging & progress handling
[ -f ./libotautils5 ] && source ./libotautils5


otautils_update_progressbar

# From v0.13.N
logmsg "I" "install" "" "removing deprecated ft lib extra copy"
[ -f "/mnt/us/${HACKNAME}/lib/bci/libfreetype.so.6" ] && rm -rf "/mnt/us/${HACKNAME}/lib/bci/libfreetype.so.6"

# Install our hack's custom content
# But keep the user's custom content...
if [ -d /mnt/us/${HACKNAME} ] ; then
    logmsg "I" "install" "" "our custom directory already exists, checking if we have custom content to preserve"
    # If we disabled autoreboot, don't re-enable it...
    if [ ! -f /mnt/us/${HACKNAME}/autoreboot ] ; then
        HACK_EXCLUDE="${HACK_EXCLUDE} ${HACKNAME}/autoreboot"
        logmsg "I" "install" "" "keep autoreboot status (off)"
    fi
fi

otautils_update_progressbar

# Okay, now we can extract it. Since busybox's tar is very limited, we have to use a tmp directory to perform our filtering
logmsg "I" "install" "" "installing custom directory"
# Make sure our xzdec binary is executable first...
chmod +x ./xzdec
./xzdec ${HACKNAME}.tar.xz | tar -xvf -
# Do check if that went well
_RET=$?
if [ ${_RET} -ne 0 ] ; then
    logmsg "C" "install" "code=${_RET}" "failed to extract custom directory in tmp location"
    return 1
fi

otautils_update_progressbar

cd src
# And now we filter the content to preserve user's custom content
for custom_file in ${HACK_EXCLUDE} ; do
    if [ -f "./${custom_file}" ] ; then
        logmsg "I" "install" "" "preserving custom content (${custom_file})"
        rm -f "./${custom_file}"
    fi
done
# Finally, unleash our filtered dir on the live userstore
cp -af . /mnt/us/
_RET=$?
if [ ${_RET} -ne 0 ] ; then
    logmsg "C" "install" "code=${_RET}" "failure to update userstore with custom directory"
    return 1
fi
cd - >/dev/null
rm -rf src

otautils_update_progressbar

# Setup startup script
logmsg "I" "install" "" "installing upstart job"
cp -f ${HACKNAME}.conf /etc/upstart/${HACKNAME}.conf

otautils_update_progressbar

# Setup fontconfig
logmsg "I" "install" "" "installing fontconfig config"
cp -f 75-linkfonts-altfonts.conf /etc/fonts/conf.d/75-linkfonts-altfonts.conf

otautils_update_progressbar

# Generating Python bytecode for the menu rebuild script
logmsg "I" "install" "" "generating python bytecode"
# We need python, of course ;)
if [ -f "/mnt/us/python/bin/python2.7" ] ; then
    /mnt/us/python/bin/python2.7 -m py_compile /mnt/us/extensions/linkfonts-ovr/bin/rebuild_menu.py
    /mnt/us/python/bin/python2.7 -O -m py_compile /mnt/us/extensions/linkfonts-ovr/bin/rebuild_menu.py
fi

otautils_update_progressbar

logmsg "I" "install" "" "cleaning up"
rm -f ${HACKNAME}.conf ${HACKNAME}.tar.xz xzdec

otautils_update_progressbar

logmsg "I" "install" "" "done"

otautils_update_progressbar

return 0
